function [Xret,Ass,Bss] = eq3( M, C, K, time, acc_x, acc_y, force_dist)
%
% [X,A,B] = eq3( M, C, K, time, acc_x, acc_y)
%
% Computes the response of the three-dimensional dynamic system
% with mass, damping, and stiffness matrices M, C, and K respectively.
%
% It is assumed that the matrices are formulated for a system with three
% degrees of freedom per story and the displacement vector is
% [ ... X_i Y_i T_i ... ] with X_i and Y_i representing the displacement
% of the story in the x- and y-directions respectively; T_i is the rotation.
%
% For an undamped system pass an empty matrix for C, e.g.
%
%     eq3( M, [], K, time, acc_x, acc_y);
%
% time is a vector of time values matching the acceleration ground records
% `acc_x' and `acc_y', in the x- and y-directions respectively.
%
% A second form using data structures is also supported.
%
% [X,A,B] = eq3( data_structure)
%
% The structure must have the field names as given above:  M, C, ..., acc_y.
%
% In addition, data_structure may have the following fields:
%
%   Bss = The state-space `B' matrix that determines how the ground
%     accelerations map to the structural masses.  If Y is the state vector,
%     then the state space dynamic system is given by
%    
%        Y'(t) = A Y(t) + Bss acc(t)
%    
%     where ' indicates time differentiation and `acc' is the ground
%     accelerations, given by:
%
%     acc(t) = [acc_x(t); acc_y(t)]
%
%     A is formed automatically using M, C, K.    Note that the dimensions
%     of Y are 2n x 1, where n is the dimension of the mass matrix M.
%     States 1 through n (of Y) are the displacements, while states n+1
%     through 2n are the velocities.  Thus, Bss must have dimension 2n x 2.
%     In almost all cases, Bss will be be a n x 2 zero matrix stacked on top
%     of an n x 2 position matrix.  The position matrix will have negative
%     ones in the translation entries and will be multiplied by the inverse
%     of the mass matrix.  If Bss is not given it is formed assuming that a
%     condensed system is being used with three degrees of freedom for each
%     story: translations in the x- and y-direction and rotation.  The
%     default Bss matrix then maps acc_x forces to the first degree of
%     freedom per story, acc_y forces to the second, and nothing to the
%     third degree of freedom.
%
%   Css = The state-space `C' matrix.  This is used to determine the
%     quantities which are returned:
%
%         X(t) = Css Y(t)
%
%     Thus, Css must have dimensions m x 2n where m is the number of states
%     that are to be returned.  If Css is not provided, the full state is
%     returned.
%
%   time_slice = Used to break up the analysis into time slices (usually
%     because of memory issues).  time_slice is a vector with time slices to
%     analyze.  The time slices will usually start with 0 and should be
%     continous.
%     Note:  If a custom Css is given, then extra memory will be required for
%     the time_slice for the solution and a full state matrix
%
%   anal_dt = The time step to use during analysis.  The simulation will be
%     run at this sampling interval.  This field must be defined when using
%     time_slice and should be less than the sampling interval of the original
%     time series (acc_x & acc_y).
%
% Output
%
% Returns the response of the structure to the ground acceleration record
% acc_x and acc_y.

% force_dist not implemented yet.
% force_dist is an optional force distribution matrix indicating the
% location of seismic forces.  It should show where the forces act on the
% structure.  If this argument is not given, it will be assumed that a
% condensed system is being used with three degrees of freedom for each
% story:  translations in the x- and y-direction and rotation.  The 

struct_val = 0;   % set to true if a structure was passed in.
if isstruct(M)
  struct_val = 1;
	s = M;
	M = s.M;
	C = s.C;
	K = s.K;
	time = s.time;
	acc_x = s.acc_x;
	acc_y = s.acc_y;
elseif nargin < 6
	error( 'Usage: eq3( M, C, K, time, acc_x, acc_y).  See help for details.');
end

Ass = dif2ss( M, C, K);

% X' = AX + BU

if struct_val & isfield( s, 'Bss')
	Bss = s.Bss;
else
	Bss = zeros( size(M,1) *2, 2);
	for x = size(M,1)+1 : 3 : size(M,1)*2
		midx = x - size(M,1);
		Bss(x,1)   = -1/M(midx  ,  midx);
		Bss(x+1,2) = -1/M(midx+1,midx+1);
	end
end

if struct_val & isfield( s, 'Css')
	Css = s.Css;
	full_state_feedback = 0;
else
	% full state feedback
	Css = eye(size(M)*2);
	full_state_feedback = 1;
end

Dss = zeros( size(Css,1), size(Bss,2));

% Must pass row vectors to lsim
% When used with Ass,Bss,Css,Dss, lsim makes sure the return time
% value is the same as the one passed in.  I don't know if
% this is a bug or a feature...
if size(acc_x,1) > size(acc_x,2)
	acc = [acc_x'; acc_y'];
else
	acc = [acc_x;  acc_y ];
end
clear acc_x acc_y;  % clear these from local space to save memory
if struct_val & isfield( s, 'time_slice')
	% We are breaking up the analysis into time slices because of memory
	% issues.
	time_beg = s.time_slice(1);
	Xind = 1;   % current index into the return array
	% X0 is the initial state, assumed to be zero
	X0 = zeros( size(Ass,2), 1);
	% It is more convenient to make this a row vector right away.
	for time_end = s.time_slice(2:end)
		% Current time slice indexes
		cur_ind = find( time >= time_beg & time <= time_end);
		% set up the new time slice
		time_slice = time(cur_ind(1)):s.anal_dt:time(cur_ind(end));
		% interp1 needs column vectors, will need to transpose later
		acc_slice  = interp1(time(cur_ind), acc(:,cur_ind)', time_slice, '*linear');
		if full_state_feedback
			X = lsim( Ass, Bss, Css, Dss,  ...
				acc_slice', time_slice - time_slice(1), X0);
			X0 = X(end,:);
		else
			[X,t,Y] = lsim( Ass, Bss, Css, Dss,  ...
				acc_slice', time_slice - time_slice(1), X0);
			X0 = Y(end,:);
		end
		size(interp1(time_slice, X, time(cur_ind)))
		Xret(cur_ind,:) = interp1(time_slice, X, time(cur_ind));
		time_beg = time_end
	end
else
		Xret = lsim( Ass, Bss, Css, Dss, acc, time);
end

