function [time, data_x, data_y, angle_x, angle_y] = ground_record( event, station, options)
% Synopsis:
%
% events   = ground_record( )
% stations = ground_record( event)
% [time, data_x, data_y, angle_x, angle_y] = ground_record( event, station, options)
%
% Loads an earthquake ground record for a seismic event and station.
%
% Called with no arguments, returns a matrix of event names.
%
% Called with one argument (the event name) returns station names available
%
% Called with two or more arguments returns vectors with the time, x- and
% y-direction components of the ground motion.  For this case the arguments
% are:
%
% event    - The event name
% station  - The station name
% options  - An optional structure argument specifying options to transform or
%            scale the output.
%            type     - Type of record to return, "acc" for ground
%                       acceleration, "vel" for ground velocity, "dis" for
%                       gournd displacement.  Default: "acc"
%                       ** THIS IS NOT IMPLEMENTED YET ** The ground acceleration
%                       is returned in all cases.
%            rotation - Angle to rotate the record from the recorded locations
%                       Default: 0 This argument can also be one of the
%                       following strings: "max" - rotate so that the
%                       max(abs(data_x)) is maximized.  "max-peak-to-peak" -
%                       rotate so that max(data_x) - min(data_x) is maximized.
%                       ** THIS IS NOT IMPLEMENTED YET **
%            time     - Time values for which data is requested.  Must be in
%                       ascending order and any values less than zero or larger
%                       than the maximum time of the record will be truncated.
%            eq_scale  - Scalar value to scale the earthquake records.
%                       If x and y are the full scale x- and y-components,
%                       respectively, then data_x = options.eq_scale * x, data_y
%                       = options.eq_scale * y.  Default is 1.00.
%            mint     - Scalar value for the minimum time.
%                       The data and time vectors will be truncated so that
%                       min(time) >= mint
%            maxt     - Scalar value for the maximum time.
%                       The data and time vectors will be truncated so that
%                       max(time) <= maxt
%            units    - One of the following strings indicating the unit desired
%                       'inches' - Use inches for the lenght unit.  Thus
%                                  accelerations = inch/sec^2, velocity = inch/sec
%                                  displacement = inch.
%           
% The return values when called with more than 2 input arguments are:
%
% time    - Row vector with time values of the record.
% data_x  - Ground motion row vector in the "x-direction"
% data_y  - Ground motion row vector in the "y-direction".  This will be
%           90 degrees counter-clockwise from data_x.
% angle_x - The angle from north of the x-record.
% angle_y - The angle from north of the y-record.
%
% Note:  All ground records start at time 0, thus by specifying a minimum
%        time the data returned will begin mint seconds after the start of
%        the record.


% Linear dynamic problem to check how ODE suite tolerances affect
% the accuracy and efficiency of the nonlinear code

if nargin < 1
	% return a cell array of event names
	time = station_names;
elseif nargin < 2
	% return the station names for the given event.
	if iscell(event)
		event = char(event);
	end
	time = station_names(event);
else % nargin >= 2 {
	if iscell(event)
		event = char(event);
	end
	% Get all of the data
	[base, angle_x_arr, angle_y_arr] = station_names(event);
	found = 0;
	for n = 1 : length(base)
		if base{n} == station
			found = 1;
			% base_[xy] are the base name of the records, e.g. nrg000 and nrg090
			angle_x = angle_x_arr(n,1);
			angle_y = angle_y_arr(n,1);
			base_x =  [station sprintf('%03d', angle_x)];
			base_y =  [station sprintf('%03d', angle_y)];
			break;
		end
	end
	if ~ found
		error(['Could not find data for event ' event ', station ' station]);
	end

	% get the data
	load( ['groundrecs/' event '/' base_x '.mat']);
	if exist( 'acc', 'var')
		data_x = acc; clear acc;
		if exist( 'acc_dt', 'var')
			time_x = 0 : acc_dt : (length(data_x) - 1) * dt;
		elseif exist( 'dt', 'var')
			time_x = 0 : dt : (length(data_x) - 1) * dt;
		elseif exist( 'acc_time', 'var')
			time_x = acc_time;
		elseif exist( 'time', 'var')
			time_x = time;
		else
			warning( 'Could not get time values for earthquake record.'); time_x = [];
		end

		load( ['groundrecs/' event '/' base_y '.mat']);
		data_y = acc; clear acc;
		if exist( 'acc_dt', 'var')
			time_y = 0 : acc_dt : (length(data_y) - 1) * dt;
		elseif exist( 'dt', 'var')
			time_y = 0 : dt : (length(data_y) - 1) * dt;
		elseif exist( 'acc_time', 'var')
			time_y = acc_time;
		elseif exist( 'time', 'var')
			time_y = time;
		else
			warning( 'Could not get time values for earthquake record.'); time_x = [];
		end
	else
		[data_x, data_y, time_x, time_y] = old_data( event, base_x, base_y);
	end

	% Time series may not be the same length.
	% Determine the shorter of the two time series and use that as the base.
	min_l = min( [ length(data_x) length( data_y) ] );
	time  = time_x(1 : min_l);

	if nargin > 2  % i.e.  options were passed in
		if isfield(options, 'maxt') & options.maxt > 0
  		time = time(find(time <= options.maxt));
		end
	end

	data_x  = data_x(1:length(time));
	data_y  = data_y(1:length(time));

	if nargin > 2
		% Check for minimum value
		if isfield(options, 'mint') & options.mint > 0
			data_index = find(time >= options.mint);
			time = time(data_index);
			data_x = data_x(data_index);
			data_y = data_y(data_index);
		end
		if isfield(options, 'eq_scale')
			data_x = data_x * options.eq_scale;
			data_y = data_y * options.eq_scale;
		end
		if isfield(options, 'time')
			% Truncate the time vector to fit (min(time),max(time))
			new_time = options.time(find(options.time >= time(1)));
			new_time = new_time(find(new_time <= time(length(time))));

			% now, interpolate to the new time value
			data_x = interp1( time, data_x, new_time);
			data_y = interp1( time, data_y, new_time);
			time = new_time;
		end

		% Note:  All time history records in the database have units cm and sec
		if isfield(options, 'units')
			switch lower(options.units)
				case {'inches','inch'}
					data_x = data_x / 2.54;
					data_y = data_y / 2.54;
				otherwise
					warning(['Unrecognized unit: ' options.units ]);
			end
		end
	end

end %  nargin >= 2 }

% Load an old format record
function [data_x, data_y, time_x, time_y] = old_data( event, base_x, base_y)
	load( ['groundrecs/' event '/' base_x '.mat']);
	load( ['groundrecs/' event '/' base_y '.mat']);
	eval([ 'data_x = ' base_x '_acc;']);
	eval([ 'data_y = ' base_y '_acc;']);

	check_var1 = [ base_x '_acc_dt'];
	check_var2 = [ base_x '_dt'];
	if exist(check_var1)
		dt = eval(check_var1);
		time_x = 0 : dt : (length(data_x) - 1) * dt;
	elseif exist(check_var2)
		dt = eval(check_var2);
		time_x = 0 : dt : (length(data_x) - 1) * dt;
	else
		eval([ 'time_x = ' base_x '_time;']);
	end

	check_var1 = [ base_y '_acc_dt'];
	check_var2 = [ base_y '_dt'];
	if exist(check_var1)
		dt = eval(check_var1);
		time_y = 0 : dt : (length(data_y) - 1) * dt;
	elseif exist(check_var2)
		dt = eval(check_var2);
		time_y = 0 : dt : (length(data_y) - 1) * dt;
	else
		eval([ 'time_y = ' base_y '_time;']);
	end
