function [T, xi, eigvecs] = modaldata( s, n)
%
%  Synopsis:
%       [T, xi, eigvecs] = modaldata( s, n)
%
%  Returns the modal period(s) (`T'), damping factor(s) (`xi'), and
%  eigenvector(s) (`eigvecs') for mode(s) `n' of system structure `s'.  The `n'
%  argument is optional and can be a scalar or vector quantity.  If `n' is not
%  given, periods and damping factors for all modes will be returned.
%
%  `s' must have the following fields:
%
%   M    - system mass matrix.
%   C    - system damping matrix.
%   K    - system stiffness matrix.
%
%   Note:  If s.pre_yield_K is defined (e.g. for nonlinear systems), then
%          it is used instead of s.K

if nargin < 2
	n = 1:size(s.M,1);
end

if isfield( s, 'pre_yield_K')
	[eigvecs,vals] = difeig( s.M, s.pre_yield_K);
else
	[eigvecs,vals] = difeig( s.M, s.K);
end
periods = eig2period(vals);

omega = sqrt(vals(n));
T = periods(n);

if length(n) > 1
	xi = zeros(length(n),1);
	for x = 1:length(n)
		xi(x) = eigvecs(:,n(x))' * s.C * eigvecs(:,n(x)) ...
		      / (2*omega(x) * eigvecs(:,n(x))' * s.M * eigvecs(:,n(x)));
	end
else
	xi = eigvecs(:,n)' * s.C * eigvecs(:,n) / ...
	     (2*omega * eigvecs(:,n)' * s.M * eigvecs(:,n));
end

% return only the needed eigenvectors
eigvecs = eigvecs(:,n);

